package com.virjar.nat.client;

import com.virjar.nat.ALOG;
import com.virjar.nat.client.listener.ProxyChannelBorrowListener;
import com.virjar.nat.protocol.Constants;

import java.util.Iterator;
import java.util.Map;
import java.util.Map.Entry;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentLinkedQueue;

import io.netty.bootstrap.Bootstrap;
import io.netty.buffer.Unpooled;
import io.netty.channel.Channel;
import io.netty.channel.ChannelFuture;
import io.netty.channel.ChannelFutureListener;
import io.netty.channel.ChannelOption;
import io.netty.util.AttributeKey;

/**
 * 代理客户端与后端真实服务器连接管理
 *
 * @author fengfei
 */
public class ClientChannelMannager {


    private static final AttributeKey<Boolean> USER_CHANNEL_WRITEABLE = AttributeKey.newInstance("user_channel_writeable");

    private static final AttributeKey<Boolean> CLIENT_CHANNEL_WRITEABLE = AttributeKey.newInstance("client_channel_writeable");

    private final int MAX_POOL_SIZE = 100;

    private Map<String, Channel> littleProxyServerChannels = new ConcurrentHashMap<String, Channel>();

    private ConcurrentLinkedQueue<Channel> proxyChannelPool = new ConcurrentLinkedQueue<Channel>();

    private volatile Channel cmdChannel;

    private String serverHost;

    public ClientChannelMannager(String serverHost) {
        this.serverHost = serverHost;
    }

    public void borrowProxyChanel(Integer port, Bootstrap join2NatServerBootStrap, final ProxyChannelBorrowListener borrowListener) {
        Channel channel = proxyChannelPool.poll();
        if (channel != null) {
            borrowListener.success(channel);
            return;
        }

        join2NatServerBootStrap.connect(serverHost, port).addListener(new ChannelFutureListener() {

            @Override
            public void operationComplete(ChannelFuture future) throws Exception {
                if (future.isSuccess()) {
                    borrowListener.success(future.channel());
                } else {
                    ALOG.e(Constants.tag, "connect proxy server failed", future.cause());
                    borrowListener.error(future.cause());
                }
            }
        });
    }


    public void returnProxyChanel(Channel proxyChanel) {
        if (proxyChannelPool.size() > MAX_POOL_SIZE) {
            proxyChanel.close();
        } else {
            proxyChanel.config().setOption(ChannelOption.AUTO_READ, true);
            proxyChanel.attr(Constants.NEXT_CHANNEL).remove();
            proxyChannelPool.offer(proxyChanel);
            ALOG.i(Constants.tag, "return ProxyChanel to the pool, channel is :" + proxyChanel + ", pool size is : " + proxyChannelPool.size());
        }
    }

    public void removeProxyChanel(Channel proxyChanel) {
        proxyChannelPool.remove(proxyChanel);
    }

    public void setCmdChannel(Channel cmdChannel) {
        this.cmdChannel = cmdChannel;
    }

    public Channel getCmdChannel() {
        return cmdChannel;
    }

    public void setRealServerChannelUserId(Channel littleProxyServerChannel, String userId) {
        littleProxyServerChannel.attr(Constants.USER_ID).set(userId);
    }

    public String getRealServerChannelUserId(Channel realServerChannel) {
        return realServerChannel.attr(Constants.USER_ID).get();
    }

    public Channel getRealServerChannel(String userId) {
        return littleProxyServerChannels.get(userId);
    }

    public void addLittleProxyServerChannel(String userId, Channel realServerChannel) {
        littleProxyServerChannels.put(userId, realServerChannel);
    }

    public Channel removeRealServerChannel(String userId) {
        return littleProxyServerChannels.remove(userId);
    }

    public boolean isRealServerReadable(Channel realServerChannel) {
        return realServerChannel.attr(CLIENT_CHANNEL_WRITEABLE).get() && realServerChannel.attr(USER_CHANNEL_WRITEABLE).get();
    }

    public void clearRealServerChannels() {
        ALOG.w(Constants.tag, "channel closed, clear real server channels");

        Iterator<Entry<String, Channel>> ite = littleProxyServerChannels.entrySet().iterator();
        while (ite.hasNext()) {
            Channel realServerChannel = ite.next().getValue();
            if (realServerChannel.isActive()) {
                realServerChannel.writeAndFlush(Unpooled.EMPTY_BUFFER).addListener(ChannelFutureListener.CLOSE);
            }
        }

        littleProxyServerChannels.clear();
    }
}
